@@ -21,7 +21,7 @@ This module implements generic functions for subscriptions
 """
 __version__ = "$Revision: 61009 $"
 # $Source$
-# $Id: SubscriptionTool.py 241646 2011-06-29 16:53:14Z yboussard $
+# $Id: SubscriptionTool.py 241667 2011-06-29 22:19:01Z yboussard $
 __docformat__ = 'restructuredtext'
 
 # Python imports
@@ -45,6 +45,7 @@ from Products.CMFCore.tests.base.security import PermissiveSecurityPolicy
 from Products.CMFCore  import permissions as CMFCorePermissions
 from Products.CMFCore.utils import getToolByName, UniqueObject
 from Products.CMFPlone.PropertiesTool import SimpleItemWithProperties
+from Products.statusmessages.interfaces import IStatusMessage
 
 # Archetypes imports
 from Products.Archetypes.public import *
@@ -770,6 +771,21 @@ A change related to your subscription to "%(subscription_title)s" was found:
 """
         return template
 
+    security.declareProtected(SubscriptionPermissions.ManageSubscriptionContent, 'updateDateForAllSubscriptions')
+    def updateDateForAllSubscriptions(self):
+        """ usefull when you don't want send mail ,
+        this method update subscription for all user """
+        now = DateTime()
+        subscriptions = self.getUpdatedSubscriptions(now)
+        for subscription in self.getUpdatedSubscriptions(now):
+            subscription.setLast_send(now)
+        try:
+            IStatusMessage(self.REQUEST).addStatusMessage('Updated %d subscriptions' % len(subscriptions),
+                                                          type="info")
+            return self.subscribers_management()
+        except:
+            pass
+
 
     security.declareProtected(SubscriptionPermissions.ViewSubscriptionContent, 'getUpdatedSubscriptions')
     def getUpdatedSubscriptions(self, date):
@@ -786,6 +802,41 @@ A change related to your subscription to "%(subscription_title)s" was found:
             subscriptions.append(subscription.getObject())
         return subscriptions
 
+    security.declareProtected(SubscriptionPermissions.ManageSubscriptionContent, 'simulateMailing')
+    def simulateMailing(self):
+        """ usefull when you don't want send mail ,
+        and you want to see what do the subcriber tool"""
+
+        r = self.mailing(preview=1)
+        try:
+            if r:
+                for (i,mail) in enumerate(r):
+                    IStatusMessage(self.REQUEST).addStatusMessage("%d : %s" %
+                                                                  (i,unicode(mail,'utf8','ignore')),
+                                                                  type="info")
+            else:
+                
+                IStatusMessage(self.REQUEST).addStatusMessage('No mail to be send',
+                                                              type="info")
+            return self.subscribers_management()
+        except:
+            logger.exception('there is an error')
+            pass
+
+    security.declareProtected(SubscriptionPermissions.ManageSubscriptionContent, 'doMailing')
+    def doMailing(self):
+        """ fire the subcriber tool"""
+
+        self.mailing()
+        try:
+            IStatusMessage(self.REQUEST).addStatusMessage("Mailling done")
+           
+            return self.subscribers_management()
+        except:
+            logger.exception('there is an error')
+            pass
+
+
     security.declareProtected(SubscriptionPermissions.ManageSubscriptionContent, 'mailing')
     def mailing(self, preview=0):
         """Searches for all subscriptions that haven't been sent according to
@@ -832,7 +883,7 @@ A change related to your subscription to "%(subscription_title)s" was found:
             Log(LOG_DEBUG, 'update last send date: ', is_one_mail_sent)
             if is_one_mail_sent:
                 subscription.setLast_send(now)
-
+            
         if preview:
             return mails
         else:
@@ -877,7 +928,7 @@ A change related to your subscription to "%(subscription_title)s" was found:
                 validation = False # perhaps a user from Zope root acl_users?
 
             # Check if member has an email
-            if member.getProperty('email') in (None, ''):
+            if member and member.getProperty('email') in (None, ''):
                 validation = False
         else:
             validation = False
